# (C) Copyright 2008 Brian Fisher, Zach Young, James Paige
# This is Free Software released under the terms of the GPL
# Created for the Pyweek May 2008 competition: "ROBOTS!"

from locals import *
from random import randint, choice

from city import CityBlock

#------------------------------------------------------------------------------

class ThankYouStyle(AssetBase):
    image = AssetRef(ImageAsset, "!")

#------------------------------------------------------------------------------

class CustomerData(AssetBase):
    help_image = AssetRef(ImageAsset, "interface/balloon_help")
    thank_you_images = AssetList(AssetObject(ThankYouStyle))
    thank_you_time = AssetFloat(5.0)
    furthest = AssetInteger(10, "blocks", 3, 100, "Caller location will not be further away than this")
    time_limit = AssetFloat(2.0, "minutes", 0.01, 60*24*365)
    
#------------------------------------------------------------------------------

class Customer(object):

  style = AssetRef(CustomerData)
  
  def __init__(self):
    self.name = self.new_random_name()
    self.where = self.new_random_location()
    self.time_on_hold = 0.0
    self.thank_you_time = .0

  def update(self, seconds):
    if self.thanking():
        self.thank_you_time -= seconds
        if self.thank_you_time < .0:
            self.finish_completion()
    else:
        self.time_on_hold += seconds
        game = HamsterWheel.the_game
        if (game.robot.pos - self.pixel_pos()).length < 250:
          self.start_completion()

  def pixel_pos(self):
    block_offset = CityBlock.size + CityBlock.street_size
    result = self.where * block_offset
    if self.where.y % 2 == 1:
      result.x -= block_offset.x / 2
    return result

  def draw(self, display):
    game = HamsterWheel.the_game
    if self.thanking():
      if len(self.style.thank_you_images) > 0:
        balloon_index = self.satisfaction(len(self.style.thank_you_images)-1)
        balloon_index = max(balloon_index, 0)
        image = self.style.thank_you_images[balloon_index].image
        display.render(image, self.pixel_pos() - game.camera)
    else:
      display.render(self.style.help_image, self.pixel_pos() - game.camera)
      
  def thanking(self):
    return self.thank_you_time > .0
    
  def satisfaction(self, out_of=5):
    time_in_seconds = self.style.time_limit * 60.0
    return out_of - int(out_of / time_in_seconds * self.time_on_hold)

  def time_string(self):
    return "%2d:%02d" % (self.time_on_hold / 60, self.time_on_hold % 60) 

  def where_in_pixels(self):
    size = CityBlock.size + CityBlock.street_size
    return self.where * size

  def new_random_name(self):
    f = open("data/names.txt")
    all = f.readlines()
    name = choice(all).strip() + " " + choice(all).strip()
    return name

  def new_random_location(self):
    while True:
      loc = vec2d(randint(-self.style.furthest, self.style.furthest), randint(-self.style.furthest,self.style.furthest))
      if abs(loc.x) < self.style.furthest / 3 and abs(loc.y) < self.style.furthest / 3:
        continue
      break
    return loc

  def start_completion(self):
    game = HamsterWheel.the_game
    game.play_completion()
    self.thank_you_time = self.style.thank_you_time

  def finish_completion(self):
    self.thank_you_time = .0
    self.time_on_hold = 0.0
    self.where += self.new_random_location()
    self.name = self.new_random_name()

#------------------------------------------------------------------------------

__BrokenThankYouStyle = ThankYouStyle()
RegisterAssetExtension(ThankYouStyle, "thankyou", __BrokenThankYouStyle)

__BrokenCustomer = CustomerData()
RegisterAssetExtension(CustomerData, "customer", __BrokenCustomer)
