"""
A module for generic little programming effeciency stuff
currently contains enums
(C) Copyright 2005 B. Fish & Hamster Republic Productions
"""

########################################################################
class InvalidEnumValues(Exception): pass

########################################################################
class Enum( object ):
    """ this gives you a compact notation for defining a canonical set of 
    string values and naming their context. It's superior to the nifty little:
        a, b, c = ["a", "b", "c"]
    trick because the set of enumerated values is an object itself (Enum)
    which can be iterated & return a count and all that good stuff """    
    def __init__( self, *values ):
        self._values = list(values)
        for value in self._values:
            if ' ' in value:
                raise(InvalidEnumValues("ERROR: Enum values most be single words"))
            if (hasattr(self, value)):
                raise(InvalidEnumValues("ERROR: Enum values cant mask object access or contain duplicates"))
            setattr( self, value, value )
    
    def __contains__( self, item ):
        return item in self._values
    
    def __iter__( self ):
        return self._values.__iter__()
    
    def __len__( self ):
        return len(self._values)

    def __getitem__(self, key):
        return self._values[key]
    
########################################################################
## Unit Testing                                                       ##
########################################################################
if __name__ == "__main__":

    import unittest

    ####################################################################
    class UnitTestEnum(unittest.TestCase):

        def setUp(self):
            pass
        
        def makeEnum(self, enum_values):
            Enum(*enum_values)
        
        def testBadInput(self):
            self.assertRaises(InvalidEnumValues, self.makeEnum, ["__dict__"])
            self.assertRaises(InvalidEnumValues, self.makeEnum, ["SomeName", "SomeNameRepeated", "SomeNameRepeated"])
            self.assertRaises(InvalidEnumValues, self.makeEnum, ["NameNoSpaces", "Name With Spaces", "Name_With_Underscores"])

        def testComparison(self):
            myEnum = Enum("Apples", "Oranges", "NotFruit")
            ApplesValue = myEnum.Apples
            self.assert_(ApplesValue != myEnum.Oranges)
            self.assert_(ApplesValue != myEnum.NotFruit)
            self.assert_(ApplesValue == myEnum.Apples)
            OtherEnum = Enum("Cucumber", "Tomato", "NotFruit")
            self.assert_(ApplesValue != OtherEnum.Cucumber)
            self.assert_(ApplesValue != OtherEnum.Tomato)
            self.assert_(ApplesValue != OtherEnum.NotFruit)
            # just checking to make sure behavior doesn't change...
            self.assert_(myEnum.NotFruit == OtherEnum.NotFruit)

        def testEnum(self):
            PossibleValues = ["Check", "For", "Membership"]
            myEnum = Enum(*PossibleValues)
            for i in PossibleValues:
                self.assert_(i in myEnum)
            for i in myEnum:
                self.assert_(i in PossibleValues)
                PossibleValues.remove(i)
            self.assert_(len(PossibleValues) == 0)

        def testCount(self):
            myEnum = Enum("One", "Two", "Three")
            self.assert_(len(myEnum) == 3)

    ########################################################################

    unittest.main()

    ########################################################################

    
    
