"""
Text Printing and Font management
(C) Copyright 2006 Hamster Republic Productions
"""

########################################################################
import pygame.font
import pygame.surface
from motherhamster.cheekpouches import *
from motherhamster.imageassets import ImageAsset
from motherhamster import profiler
import array

from motherhamster.mathobj import vec2d

########################################################################
_EstimatedSurfaceMemory = 0

def EstimateSurfaceMemory():
    global _EstimatedSurfaceMemory
    return _EstimatedSurfaceMemory

def CountSurfaceMemory(surface):
    global _EstimatedSurfaceMemory
    size = surface.get_width()*surface.get_height()*surface.get_bytesize()
    _EstimatedSurfaceMemory += size
    return size

def SubtractSurfaceMemory(size):
    global _EstimatedSurfaceMemory
    _EstimatedSurfaceMemory -= size

########################################################################

class Image(object):
    """an image that can be rendered"""
    def __init__(self, surface, hotspot=None):
        self.surface = surface
        if hotspot == None:
            r = surface.get_rect()
            self.hotspot = vec2d(r.center)
        else:
            self.hotspot = hotspot
        global CountSurfaceMemory
        self._memory_estimate = CountSurfaceMemory(surface)
        self.blend_mode = ImageAsset.blend_modes.alpha_blend

    def __del__(self):
        global SubtractSurfaceMemory
        if (SubtractSurfaceMemory != None):
            SubtractSurfaceMemory(self._memory_estimate)

    def _get_size(self):
        return vec2d(self.surface.get_width(), self.surface.get_height())        
    size = property(_get_size, doc="gets the default pixel size of the image for the asset")

    def _get_width(self):
        return self.surface.get_width()
    width = property(_get_width, doc="gets the default pixel width of the image for the asset")
    
    def _get_height(self):
        return self.surface.get_height()
    height = property(_get_height, doc="gets the default pixel height of the image for the asset")

########################################################################
class FontAsset(AssetBase):
    """a python object representation of an Font asset
       it represents the ttf file and provides a facility for caching
       the pygame font objects for printing at a specific size"""
       
    def __init__(self, asset_path):
        AssetBase.__init__(self)
        self.asset_path = asset_path
        self.font_cache = {}
 
    def pygame_font(self, height):
        if self.asset_path == None:
            # Use the pygame default font if no other is specified, but double it, since it is tinyful
            return pygame.font.Font(None, height * 2)
        if height not in self.font_cache:
            dish = GetFoodDish(self.asset_path.root)
            #TEMP:
            native_filename = dish._get_native_filename(self.asset_path)
            self.font_cache[height] = pygame.font.Font(native_filename, height)
            #asset_file = dish.open(self.asset_path.filepath, "r")
            #self.font_cache[height] = pygame.font.Font(asset_file, height)
        return self.font_cache[height]
            
    def asset_editor_image(self):
        pyfont_title = self.pygame_font(32)
        assetid_image = pyfont_title.render(self.asset_path.assetid, True, (0,0,0), (255,255,255))
        pyfont = self.pygame_font(24)
        block_len = 24
        block_images = []
        for group_start in (32, 160):
            for block_start in xrange(group_start, group_start + 96, block_len):
                block_string = ""
                for char_index in xrange(block_start, block_start + block_len - 1):
                    block_string += chr(char_index)
                try:
                    block_image = pyfont.render(block_string, True, (0,0,0), (255,255,255))
                except:
                    block_image = pyfont.render("error on: "+str(block_start)+"-"+str(block_start + block_len - 1), True, (0,0,0), (255,255,255))
                    print "font",self.asset_path.assetid,"failed to render",block_start,block_start + block_len - 1
                block_images.append(block_image)
        final_width = reduce(lambda w, img: max(img.get_width(), w), block_images, assetid_image.get_width())
        final_height = reduce(lambda h, img: img.get_height() + h, block_images, assetid_image.get_height())
        asset_image = pygame.Surface([final_width, final_height], 0, 24)
        y = 0
        asset_image.blit(assetid_image, (0, y))
        y += assetid_image.get_height()
        for block_image in block_images:
            asset_image.blit(block_image, (0, y))
            y += block_image.get_height()
        return asset_image

########################################################################
class _FontPouch(GenericAssetPouch):
    """the base class for asset loaders"""

    def __init__(self):
        GenericAssetPouch.__init__(self, FontAsset, FontAsset(None))
        self._asset_paths = None

    def AssetExtensions(self):
        return [".ttf"]

    def _load(self, asset_path):
        """loads a ttf file into a FontAsset"""
        path = self.GetAssetPathForID(asset_path)
        if (path):
            asset = FontAsset(path)
            return asset
        return None

    def _save(self, asset_path, obj):
        """a null-op - no editable data"""
        pass

########################################################################
class GlyphImage(Image):
    """an image that can be rendered, which represents a glyph
       in a font, after all relevant filter effects have been applied"""
    def __init__(self, surface, hotspot, char_spacing):
        Image.__init__(self, surface, hotspot)
        self.spacing = char_spacing
        
########################################################################
class GlyphSet(AssetBase):
    """an asset that lets you specify font characteristics and pretty
       filters and then caches individual characters for printing"""
    
    font = AssetRef(FontAsset, "!None", "which font to use")
    height = AssetInteger(20, "pixels", 4, 1000, "height of the rendered characters (filters may cause this to be larger)")
    color = AssetColor()
    test_string = AssetString("Waltz, bad nymph... for quick jigs vex!")
    
    def __init__(self):
        AssetBase.__init__(self)
        self.glyph_cache = {}
           
    def char(self, char_index, font_size=None):
        if(font_size == None):
            font_size = self.height
        cache_key = (char_index, font_size)
        if cache_key not in self.glyph_cache:
            print "rendering: ",char_index,"height",font_size
            pyfont = self.font.pygame_font(font_size)
            raw_image = pyfont.render(char_index, True, self.color)
            spacing = raw_image.get_width()
            hotspot = vec2d(0, -pyfont.get_descent())
            (final_surface, hotspot) = self.filter_surface(raw_image, hotspot)
            self.glyph_cache[cache_key] = GlyphImage(final_surface, hotspot, spacing)
        return self.glyph_cache[cache_key]

    def char_width(self, char_index, font_size=None):
        if(font_size == None):
            font_size = self.height
        cache_key = (char_index, font_size)
        if cache_key not in self.glyph_cache:
            self.char(char_index, font_size)
        return self.glyph_cache[cache_key].width

    def char_height(self, char_index, font_size=None):
        if(font_size == None):
            font_size = self.height
        cache_key = (char_index, font_size)
        if cache_key not in self.glyph_cache:
            self.char(char_index, font_size)
        return self.glyph_cache[cache_key].height

    def char_size(self, char_index, font_size=None):
        w = self.char_width(char_index, font_size)
        h = self.char_height(char_index, font_size)
        return vec2d(w, h)

    # TODO possibly obsolete
    def typeout(self, text, display, position):
        pos = vec2d(position)
        for letter in text:
            display.render(self.char(letter), pos)
            pos.x += self.char_width(letter)

    def filter_surface(self, surface, hotspot):
        #CHECK: need to implement filters in a data directed way
        for y in xrange(surface.get_height()):
            husk = [0, 0]
            for x in xrange(surface.get_width()):
                new_value = surface.get_at((x, y))[3]
                for i in xrange(len(husk)):
                    old_value = husk[i]
                    husk[i] = new_value
                    new_value += old_value
                surface.set_at((x,y), (255,255,255,new_value/4))
        for x in xrange(surface.get_width()):
            husk = [0, 0]
            for y in xrange(surface.get_height()):
                new_value = surface.get_at((x, y))[3]
                for i in xrange(len(husk)):
                    old_value = husk[i]
                    husk[i] = new_value
                    new_value += old_value
                flipped = abs(new_value - 510)
                flipped = flipped * flipped
                flipped = 260100 - flipped
                flipped /= 1020
                surface.set_at((x,y), (flipped,flipped,flipped,min(new_value/2, 255)))
        return (surface, hotspot)
    
    def asset_editor_image(self):
        char_images = []
        for char in self.test_string:
            char_images.append(self.char(char))
        final_width = reduce(lambda w, img: img.spacing + w, char_images, 0)
        max_ascent = reduce(lambda h, img: max(img.height - img.hotspot.y, h), char_images, 0)
        max_descent = reduce(lambda h, img: max(img.hotspot.y, h), char_images, 0)
        final_height = max_ascent + max_descent
        asset_image = pygame.Surface([final_width, final_height], 0, 24)
        x = 0
        for char_image in char_images:
            y = max_ascent - (char_image.height - char_image.hotspot.y)
            asset_image.blit(char_image.surface, (x, y))
            x += char_image.spacing
        return asset_image
             
    def asset_set_property(self, key, value):
        self.glyph_cache = {}
        AssetBase.asset_set_property(self, key, value)

########################################################################

# TODO possibly obsolete
class GlyphString(object):
  
    def __init__(self, font, default_text=""):
        self.font = font
        self.text = default_text
        self._need_update = True

    def set(self, new_text):
        self._text = new_text
        self._need_update = True

    def _get_text(self):
        return self._text

    def _set_image(self, new_image):
        self._image = new_image
        self._need_update = True

    def _get_image(self):
        if self._need_update:
            self.update()
        return self._image

    def _set_font(self, new_font):
        self._font = new_font
        self._need_update = True

    def _get_font(self):
        return self._font

    def update(self):
        letters = []
        for c in self.text:
            letters.append(self.font.char(c))
        (width, height) = (0, 0)
        for letter in letters:
            height = max(height, letter.height)
            width += letter.width
        surface = pygame.Surface((width, height))
        x = 0
        for letter in letters:
            surface.blit(letter.surface, (x, 0))
            x += letter.width
        self._image = Image(surface)
        self._need_update = False

    text = property(_get_text, set)
    font = property(_get_font, _set_font)
    image = property(_get_image, _set_image)

########################################################################

class GlyphTyper(object):
    
    def __init__(self, display, font, start_text="", start_pos=(0,0)):
        self.font = font
        self.size = font.height # use default font size
        self.display = display
        self.text = start_text
        self.pos = start_pos
    
    def _set_pos(self, newpos):
        self._pos = vec2d(newpos)
    def _get_pos(self):
        return self._pos
    pos = property(_get_pos, _set_pos)

    def _set_text(self, newtext):
        if newtext != None:
            self._text = newtext
    def _get_text(self):
        return self._text
    text = property(_get_text, _set_text)

    def each_letter(self):
        return GlyphTyperIterator(self)

    def draw(self, settext=None, setpos=None):
        self.text = settext
        if setpos != None:
            self.pos = setpos
        for letter in self.each_letter():
            letter.draw()
    
    def width(self, settext=None):
        self.text = settext
        result = 0
        for letter in self.each_letter():
            result += letter.size.x
        return result
    
    def height(self, settext=None):
        self.text = settext
        result = 0
        for letter in self.each_letter():
            result = max(result, letter.size.y)
        return result
    
class GlyphTyperIterator(object):

    def __init__(self, glyphtyper):
        self.typer = glyphtyper
        self.index = 0
        self.pos = vec2d(self.typer.pos)

    def __iter__(self):
        return self
        
    def next(self):
        if self.index >= len(self.typer.text):
            raise StopIteration
        ascii = self.typer.text[self.index]
        letter = GlyphTyperLetter(self.typer.display, self.typer.font, ascii, self.pos, self.typer.size)
        self.pos.x += letter.size.x
        self.index += 1
        return letter

class GlyphTyperLetter(object):
    
    def __init__(self, display, font, ascii, pos, font_size=None):
        self.display = display
        self.font = font
        self.font_size = font_size
        self.ascii = ascii
        self.size = self.font.char_size(ascii, self.font_size)
        self.pos = vec2d(pos)
        
    def draw(self):
        self.display.render(self.font.char(self.ascii, self.font_size), self.pos)
        

########################################################################

fontpouch = _FontPouch()
RegisterPouch(fontpouch)

__BrokenGlyphSetReference = GlyphSet()
RegisterAssetExtension(GlyphSet, "glyphset", __BrokenGlyphSetReference)

profiler.RegisterFunction(GlyphSet, "filter_surface")

if __name__ == "__main__":
    #TODO: Fontpouch unit tests
    pass
