# (C) Copyright 2008 Brian Fisher, Zach Young, James Paige
# This is Free Software released under the terms of the GPL
# Created for the Pyweek May 2008 competition: "ROBOTS!"

from locals import *

#------------------------------------------------------------------------------

class SlantedRect(object):

  def __init__(self, size=vec2d(60,30), pos=vec2d(0,0)):
    self.size = size
    self.pos = pos

  def corners(self, pixel_pos):
    offset = self.pos + pixel_pos
    bl = vec2d(offset.x - (self.size.x + self.size.y) / 2, offset.y - self.size.y / 2)
    br = vec2d(bl.x + self.size.x, bl.y)
    tr = vec2d(br.x + self.size.y, br.y + self.size.y)
    tl = vec2d(tr.x - self.size.x, tr.y)
    return (bl, br, tr, tl)

  def is_inside(self, pixel_pos, point):
    (bl, br, tr, tl) = self.corners(pixel_pos)
    if point.y >= bl.y and point.y <= tl.y:
      if point.x >= bl.x + abs(point.y - bl.y) and point.x <= tr.x - abs(point.y - tr.y):
        return True
    return False

  def get_closest_corner(self, pixel_pos, other_pos):
    closest_dist = 9e9
    offset = other_pos - pixel_pos
    for corner in self.corners(pixel_pos):
      dist_squared = other_pos.get_dist_sqrd(corner)
      if dist_squared < closest_dist:
        closet = corner
        closest_dist = dist_squared
    return closet

  def get_ellipse_intersection_dir(self, ellipse_center_offset, ellipse_radius):
    # first lets work pretending the slanted rect is a normal rect at the origin...
    slanted_offset = vec2d(ellipse_center_offset.x - ellipse_center_offset.y, ellipse_center_offset.y)
    half_size = self.size/2.0
    aligned_on_horizontal = slanted_offset.y >= -half_size.y and slanted_offset.y <= half_size.y
    aligned_on_vertical = slanted_offset.x >= -half_size.x and slanted_offset.x <= half_size.x
    if aligned_on_horizontal and aligned_on_vertical:
      return ellipse_center_offset.normalized()
    elif aligned_on_horizontal:
      total_width = ellipse_radius.x + half_size.x
      if slanted_offset.x < 0 and slanted_offset.x > -total_width:
        return vec2d(-1,1).normalized()
      elif slanted_offset.x > 0 and slanted_offset.x < total_width:
        return vec2d(1,-1).normalized()
    elif aligned_on_vertical:
      total_height = ellipse_radius.y + half_size.y
      if slanted_offset.y < 0 and slanted_offset.y > -total_height:
        return vec2d(0,-1)
      elif slanted_offset.y > 0 and slanted_offset.y < total_height:
        return vec2d(0,1)
    else:
        x_dist_corner = abs(slanted_offset.x) - half_size.x
        y_dist_corner = abs(slanted_offset.y) - half_size.y
        if (x_dist_corner/ellipse_radius.x)**2 + (y_dist_corner/ellipse_radius.y)**2 <= 1:
          quadrant_sign = slanted_offset/abs(slanted_offset)
          collision_dir = quadrant_sign*vec2d(x_dist_corner, y_dist_corner)
          # go back from slanty space
          collision_dir.x += collision_dir.y
          collision_dir.normalize()
          return collision_dir
      
    return None

#------------------------------------------------------------------------------
